﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Sandbox;
using Sandbox.AppCode;
using Sandbox.Common;
using Sandbox.Definitions;
using Sandbox.Engine;
using Sandbox.Game;
using Sandbox.Graphics;
using Sandbox.ModAPI;

using Sandbox.Common.ObjectBuilders;

using Sandbox.ModAPI.Interfaces;
using VRage.Game;
using VRage.Game.Components;
using VRage.ObjectBuilders;
using Sandbox.Game.Multiplayer;
using Sandbox.Game.Entities;
using Sandbox.Game.Entities.Cube;
using VRage.Game.ModAPI;

namespace MySEMods
{
    [MyEntityComponentDescriptor(typeof(MyObjectBuilder_UpgradeModule), new string[] { "LargeHackingBlock", "SmallHackingBlock" })]
    public class HackingBlock : MyGameLogicComponent
    {
        IMyFunctionalBlock m_hackingblock;
        static Random m_random = new Random();
        public override void Init(MyObjectBuilder_EntityBase objectBuilder)
        {
            m_hackingblock = Entity as IMyFunctionalBlock;
            Entity.NeedsUpdate |= VRage.ModAPI.MyEntityUpdateEnum.EACH_100TH_FRAME;
        }

        public override MyObjectBuilder_EntityBase GetObjectBuilder(bool copy = false)
        {
            return null;
        }

        public override void UpdateAfterSimulation100()
        {
            if (!MyAPIGateway.Multiplayer.IsServer)
                return;

            if (!m_hackingblock.IsFunctional || !m_hackingblock.IsWorking || !m_hackingblock.Enabled)
                return;
            if (m_hackingblock.OwnerId == 0)
                return;

            List<IMySlimBlock> blocks = new List<IMySlimBlock>();
            (m_hackingblock.CubeGrid as IMyCubeGrid).GetBlocks(blocks);
            randomize(ref blocks);
            foreach (var block in blocks)
            {
                if (attempHack(block, m_hackingblock.OwnerId))
                    return;
            }
        }

        private bool attempHack(IMySlimBlock block, long newOwner)
        {
            if (block == null || block.FatBlock == null)
                return false;

            if (!block.FatBlock.IsWorking || !block.FatBlock.IsFunctional)
                return false;

            if (block.FatBlock.GetUserRelationToOwner(newOwner) == MyRelationsBetweenPlayerAndBlock.Enemies)
            {
                if (m_random.Next() % getComputerCount(block as MySlimBlock) == 0)
                {
                    string message = Convert.ToString(block.FatBlock.EntityId) + ":" + Convert.ToString(newOwner);
                    (block.FatBlock as MyCubeBlock).ChangeOwner(0, MyOwnershipShareModeEnum.Faction);
                    (block.FatBlock as MyCubeBlock).ChangeBlockOwnerRequest(newOwner, MyOwnershipShareModeEnum.Faction);
                }
                return true;
            }
            return false;
        }

        private int getComputerCount(IMySlimBlock block)
        {
            var componets = MyDefinitionManager.Static.GetCubeBlockDefinition(block.GetObjectBuilder()).Components;
            int computers = 0;
            for (var i = 0; i < componets.Length; i++)
            {
                if (componets[i].Definition.Id.SubtypeName == "Computer")
                    computers += componets[i].Count;
            }
            return computers;
        }

        public static void randomize(ref List<IMySlimBlock> list)
        {
            int n = list.Count;
            while (n > 1)
            {
                n--;
                int k = m_random.Next(n + 1);
                IMySlimBlock value = list[k];
                list[k] = list[n];
                list[n] = value;
            }
        }
    }
}
